SystemOrganization addCategory: #'GT-BytecodeDebugger'!

GTGenericStackDebugger subclass: #GTBytecodeDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTBytecodeDebugger commentStamp: 'AndreiChis 10/22/2014 15:03' prior: 0!
I am a debugger that provides the ability to step through the execution at the bytecode level!

!GTBytecodeDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 11:08'!
defaultTitle

	^ 'Bytecode Debugger'! !

!GTBytecodeDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 10/22/2014 11:13'!
register

	self registerToolsOn: Smalltalk tools.! !

!GTBytecodeDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 10/22/2014 11:10'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry registerDebugger: self withRank: 10.! !

!GTBytecodeDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 14:17'!
sessionClass

	^ GTBytecodeDebuggerSession! !

!GTBytecodeDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 18:55'!
variablesBrowserClass

	^ GTBytecodeDebuggerVariablesBrowser! !

!GTBytecodeDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2015 22:11'!
bytecodeActionsPragmas

	^ #( bytecodeDebuggingAction )! !

!GTBytecodeDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/16/2015 15:58'!
bytecodeIn: aComposite forContext: aContext

	(self methodBytecodeWidgetIn: aComposite forContext: aContext)
		initialize: [ :bytecodePresentation |
				bytecodePresentation selection: (self session symbolicBytecodeForCurrentPCFrom: self currentContext) ];
		with: [ :bytecode | self installActionsForRootPragmas: self bytecodeActionsPragmas in: bytecode ].! !

!GTBytecodeDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/16/2015 15:58'!
bytecodePane

	^ self browser paneNamed: #bytecode! !

!GTBytecodeDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/16/2015 15:58'!
bytecodePresentation
	
	^ self bytecodePane presentations first! !

!GTBytecodeDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/16/2015 15:58'!
codePresentation
	
	^ self codePane presentations first! !

!GTBytecodeDebugger methodsFor: 'building' stamp: 'AndreiChis 1/16/2015 15:55'!
debuggerStructureIn: browser
	
	browser
		row: [ :row | 
					row
						column: #stack span: 2;
						column: #bytecode span: 2; 
						column: #code span: 3 ]
			span: 2;
		row: #inspector.
		! !

!GTBytecodeDebugger methodsFor: 'building' stamp: 'AndreiChis 1/16/2015 15:58'!
debuggerTransmissionsIn: browser

	super debuggerTransmissionsIn: browser.
	browser transmit
		from: #stack port: #selection;
		to: #bytecode;
		andShow: [ :composite :aContext | self bytecodeIn: composite forContext: aContext ]
! !

!GTBytecodeDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2015 22:20'!
debuggingActionsPragmas

	^ #()! !

!GTBytecodeDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/14/2015 23:05'!
formatBytecode: aBytecode
	| textAttributes|
	
	textAttributes := {TextColor black}.
	self selectedContext pc = aBytecode offset ifTrue: [ textAttributes := {TextColor black. TextEmphasis bold} ]. 
	self selectedContext pc > aBytecode offset ifTrue: [ textAttributes := {TextColor new color: Color darkGray} ].
	^ Text 
		string: aBytecode printString
		attributes: textAttributes! !

!GTBytecodeDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/16/2015 15:56'!
methodBytecodeWidgetIn: aComposite forContext: aContext

	^ aComposite tree
		title: 'Bytecode';
		rootsExpanded;
		display: [ aContext method symbolicBytecodes ];
		children: [ :each | each isArray ifTrue: [each] ifFalse: #() ];
		format: [ :each | self formatBytecode: each ].! !

!GTBytecodeDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/14/2015 22:15'!
stackDebuggingActionsPragmas

	^ #( bytecodeStackDebuggingAction )! !

!CopyToClipboardDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/14/2015 22:24'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			order: 35! !

!FuelOutStackDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			order: 30! !

!Context methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/16/2015 15:51'!
gtBytecodeDebuggerRetrieveContextValues
	|fieldList displayedTemp context slotsSize|

	fieldList := OrderedCollection new.
	context := self.
	context == nil ifTrue: [^('_thisContext' -> nil) asOrderedCollection].
	fieldList
		add: ('_self' ->  context receiver);
		add: ('_thisContext' ->  context).
	
	displayedTemp := OrderedCollection new.
	context stackPtr to: 1 by: -1 do: [ :index ||key|
		key := (index-1) asString.
		(index <= context tempNames size) ifTrue: [ 
			key := key, ' [', (context tempNames at: index) asString, ']'.
			displayedTemp add: (context tempNames at: index) ].
		fieldList add: (key -> (context at: index)) ].
	(context tempNames copy asOrderedCollection removeAll: displayedTemp; yourself) ifNotEmptyDo: [ :uninitializedTemps |
		uninitializedTemps do: [ :each |
			fieldList add: ('- [', each, ']') -> nil ] ].
	slotsSize := context receiver class allSlots size.
	fieldList 
		addAll: ((context receiver class allSlots reverse) collect: [:slot ||name| 
			name := slot name.
			('rcv', (slotsSize := slotsSize - 1) asString, ' [', name , ']') -> (slot read: context receiver) ]).
	
	^ fieldList! !

DebugSession subclass: #GTBytecodeDebuggerSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTBytecodeDebuggerSession commentStamp: 'AndreiChis 10/22/2014 15:03' prior: 0!
A GTBytecodeDebuggerSession provides debugging actions at the level of the bytecode
!

!GTBytecodeDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 1/16/2015 16:18'!
runToBytecode: aSymbolicBytecode inContext: aContext 
	"Attempt to step over instructions in selectedContext until the 
	execution reaches the selected instruction. This happens when the
	program counter passes the begining of selectionInterval." 
	
	aSymbolicBytecode offset <= aContext pc 
		ifTrue: [ ^self ].
	self stepOver: aContext.
	[ aContext == self interruptedContext and: [ aContext pc < aSymbolicBytecode offset ] ] 
		whileTrue: [ self stepOver: aContext ]! !

!GTBytecodeDebuggerSession methodsFor: 'private' stamp: 'AndreiChis 1/16/2015 16:15'!
stepToFirstInterestingBytecodeIn: aProcess
	"Do not step over any bytecode."
	
	^ aProcess suspendedContext! !

!GTBytecodeDebuggerSession methodsFor: 'acessing' stamp: 'AndreiChis 10/22/2014 14:47'!
symbolicBytecodeForCurrentPCFrom: aContext
	| pc |
	
	pc := aContext pc.
	^ aContext method symbolicBytecodes 
		detect: [ :each | each offset = pc ]
		ifNone: [ nil ]! !

!SymbolicBytecode methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/14/2015 17:10'!
= anObject

	self == anObject
		ifTrue: [^ true].

	(self class == anObject class)
		ifFalse: [^ false].

	^ (self bytes = anObject bytes) and: [ 
		(self description = anObject description) and: [ 
			(self method = anObject method) and: [ 
				self offset = anObject offset ] ] ]! !

!ResumeDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousPlay! !

!StepIntoDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/16/2015 16:25'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousInto;
			label: 'Step into bytecode'
			! !

!RestartDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousRestart! !

DebugAction subclass: #GTGoAndInspectBytecodeDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTGoAndInspectBytecodeDebugAction commentStamp: 'AndreiChis 10/23/2014 15:20' prior: 0!
GTGoAndInspectBytecodeDebugAction class is a debugging action that opens the currently selected bytecode in the inspector embeded within the debugger.
!

!GTGoAndInspectBytecodeDebugAction class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 10/23/2014 15:18'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtContextBytecodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/14/2015 22:12'!
defaultKeyText

	^ 'g'! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/23/2014 15:15'!
defaultLabel

	^ 'Go and inspect'! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/15/2015 16:29'!
executeAction
	
	self debugger inspect: self debugger bytecodePresentation selection  
		
! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 22:08'!
id

	^ GTGoAndInspectBytecodeDebugAction name! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 10/23/2014 15:18'!
initialize

	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTStepToBytecodeDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTStepToBytecodeDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/14/2015 22:49'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtContextBytecodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousPlay! !

!GTStepToBytecodeDebugAction methodsFor: 'testing' stamp: 'AndreiChis 1/15/2015 16:29'!
appliesToDebugger: aDebugger

	^ aDebugger session isInterruptedContextPostMortem not and: [ 
		aDebugger bytecodePresentation selection offset > aDebugger currentContext pc ]! !

!GTStepToBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 22:46'!
defaultLabel

	^ 'Step to bytecode'! !

!GTStepToBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 22:46'!
defaultOrder

	^ 10! !

!GTStepToBytecodeDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/15/2015 16:29'!
executeAction

	self session
		runToBytecode: self debugger bytecodePresentation selection 
		inContext: self currentContext! !

!GTStepToBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 22:46'!
id

	^ GTStepToBytecodeDebugAction name! !

GTDebuggerVariablesBrowser subclass: #GTBytecodeDebuggerVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTBytecodeDebuggerVariablesBrowser methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/14/2015 19:36'!
variableTypeOf: aString in: aContext
	|variableName|
	
	variableName := (aString includes: $[)
			ifTrue: [ (aString copyFrom: (aString indexOf: $[) +1 to: (aString indexOf: $])-1) trim  ]
			ifFalse: [ aString ].
	^ super 
		variableTypeOf: variableName
		in: aContext! !

!GTBytecodeDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 1/14/2015 19:07'!
variablesFromContext: aContext 

	^ aContext gtBytecodeDebuggerRetrieveContextValues! !

!GTBytecodeDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 1/14/2015 19:02'!
variablesIn: aComposite

	^ (super variablesIn: aComposite)
		sorted: nil! !

!StepOverDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/16/2015 16:29'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousOver;
			label: 'Step over bytecode'! !

!GTBrowseDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/14/2015 22:17'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtContextBytecodeStackDebuggingAction>
	
	^ super debugActionsFor: aDebugger! !

!ChangeDebuggerAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ self gtChangeDebuggerActionsFor: aDebugger! !
